import bottle
from configparser import ConfigParser
import json
import os

from logger import Logger
from singleton import Singleton


class AnorGo(metaclass=Singleton):
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса')

        conf = ConfigParser()
        conf.read(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.ini'))
        self.servicePath = conf.get('anorgo', 'path') if conf.has_option('anorgo', 'path') else '/api/v1'

        self.__updateCases()

        servicestub.route(os.path.join(self.servicePath, 'payment'), method='POST', callback=self.payment)
        servicestub.route(os.path.join(self.servicePath, 'payment/status'), method='GET', callback=self.status)
        servicestub.route(os.path.join(self.servicePath, 'payment/reversal'), method='DELETE', callback=self.reversal)
        servicestub.route(os.path.join(self.servicePath, 'payment/ofd_data/submit_qrcode'), method='POST', callback=self.fiscal)

    def payment(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('payment')

    def status(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('payment/status')

    def reversal(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('payment/reversal')

    def fiscal(self):
        body = json.loads(bottle.request.body.read())
        self.log.info("Данные запроса '%s'" % str(body))
        return self.createResponse('payment/ofd_data/submit_qrcode')

    def createResponse(self, methodName):
        if methodName not in self.cases:
            raise bottle.HTTPError(400, 'Метод "%s" не найден' % methodName)

        if not len(self.cases[methodName]):
            self.log.warn('В кейсах не осталось значений')
            return None

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodName][0]:
            return json.dumps(self.cases[methodName][0].get('response'))

        self.cases[methodName][0]['count'] -= 1
        response = json.dumps(self.cases[methodName][0].get('response'))

        if self.cases[methodName][0]['count'] == 0:
            del self.cases[methodName][0]

        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))

instance = AnorGo()
